==> sim-client-handler-4.0.1.tgz: package/index.js <==
/*
 * Copyright 2016 Brigham Young University
 *
 * Licensed under the Apache License, Version 2.0 (the "License")
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

"use strict"

const yapi          = require("./lib/yapi")
const wso2Request   = require('byu-wso2-request').request
const logger        = require("debug")("clientHandler")

const byuapiServiceDomainURI = process.env.BYUAPI_DOMAIN || 'https://api.byu.edu'

exports.clientHandler = function(configObject)
{
    const ch =
    {
        configResponse: function ()
        {
            if (this.lambda)
            {
                return {
                    statusCode: 200,
                    headers: {
                        "content-type": "application/json"
                    },
                    body: ""
                }
            }

            return {}
        },
        config: function Config(options)
        {
            if (options === undefined) options = {}

            for (const key of Object.keys(this.keys))
            {
                this.set(key, options[key], this.keys[key])
            }
            this.response = this.configResponse()
            this.policyCache = {}
            this.identity = null
        },
        validateJWT: async function ()
        {
            let self = this

            const byuJwt = require('byu-jwt')

            const jwt = self.parseJWT()
            const wellKnownUrl = self.WELL_KNOWN_URL || 'https://api.byu.edu/.well-known/openid-configuration'

            self.jwtDecoded = await byuJwt.jwtDecoded(jwt, wellKnownUrl)

            if (this.apiContext !== null && self.jwtDecoded.wso2.apiContext.indexOf(this.apiContext) !== 0 )
            {
                const err = new Error("Invalid jwt on apiContext: " + self.jwtDecoded.wso2.apiContext)
                logger(err.stack.split("\n")[1],": api context", self.jwtDecoded.wso2.apiContext, "is not found\""+this.apiContext+"\"")
                throw err
            }
        },
        send: function (statusCode, body)
        {
            if (this.lambda)
            {
                if (body === undefined)
                {
                    body = {}
                    if (statusCode >= 400)
                    {
                        let statusCodeMessage = {message: "An unknown error has occurred. Please try again."}
                        switch (statusCode)
                        {
                            case 404:
                                statusCodeMessage.message = "resource <" + this.req.resource + "> was not found"
                                break
                            case 405:
                                statusCodeMessage.message = "http Method <" + this.req.httpMethod + "> is not supported for resource <" + this.req.resource + ">"
                                break
                        }

                        body = statusCodeMessage
                    }
                }
                this.response.body = JSON.stringify(body)
                this.response.statusCode = statusCode

                return this.res.succeed(this.response)
            }

            this.response = body || {}
            this.res.status(statusCode)

            this.res.send(this.response)
        },
        body: function ()
        {
            if (this.lambda)
            {
                return JSON.parse(this.req.body)
            }
            return this.req.body
        },
        queryParams: function (asURI)
        {
            if (asURI === true)
            {
                logger("req url:", this.req.url)
                const parts = this.req.url.split("?")
                logger("req url parts:", parts, parts.length)
                if (parts.length === 2)
                {
                    logger("req query parameters:", parts[1])
                    return parts[1]
                }
                return ""
            }
            if (this.lambda)
            {
                return this.req.queryParameters
            }

            return this.req.query
        },
        pathValue: function (paramkey)
        {
            if (this.lambda)
            {
                return this.req.pathParameters[paramkey]
            }

            return this.req.params[paramkey]
        },
        parseJWT: function ()
        {
            const byuJwt = require('byu-jwt')

            let jwt = this.req.headers[byuJwt.BYU_JWT_HEADER_ORIGINAL.toLowerCase()]
            if (!jwt)
            {
                jwt = this.req.headers[byuJwt.BYU_JWT_HEADER_CURRENT.toLowerCase()]
            }
            logger("raw jwt:", jwt)

            return jwt
        },
        parseIdentity: async function ()
        {
            let self = this

            function extractIdentity(decodedJwt)
            {
                if (decodedJwt.byu.hasOwnProperty('resourceOwner'))
                {
                    self.identity = decodedJwt.byu.resourceOwner
                }
                else
                {
                    self.identity = decodedJwt.byu.client
                }
                logger("identity:", self.identity)
                return self.identity
            }

            if (self.identity !== null)
            {
                return self.identity
            }

            if (self.hasOwnProperty("jwtDecoded"))
            {
                logger("using cached decoded jwt")
                return extractIdentity(self.jwtDecoded)
            }
            else
            {
                const byuJwt = require('byu-jwt')

                const jwt = self.parseJWT()
                const wellKnownUrl = self.WELL_KNOWN_URL || 'https://api.byu.edu/.well-known/openid-configuration'

                if (jwt)
                {
                    const decodedJwt = await byuJwt.jwtDecoded(jwt, wellKnownUrl)

                    // self.req.jwt = decodedJwt
                    self.jwtDecoded = decodedJwt
                    logger("decodedJwt:", decodedJwt)

                    return extractIdentity(decodedJwt)
                }
            }

            throw new Error("unable to establish identity")
        },
        parsePolicies: async function (webResourceId)
        {
            let self = this

            if (self.policyCache[webResourceId])
            {
                return self.policyCache[webResourceId]
            }

            const response = await yapi.userPolicies(self.identity.byuId, webResourceId)

            if (response.content.length)
            {
                self.policyCache[webResourceId] = response.content
            }
            else
            {
                self.policyCache[webResourceId] = []
            }
            return self.policyCache[webResourceId]
        },
        /**
         * options web resource id and informational area are required - updateType [U] is optional
         * @param options
         * @returns boolean
         */
        authorize: async function (options)
        {
            let self = this

            const identity = await self.parseIdentity()

            if (options.allowSelf)
            {
                const selfIdValue = self.pathValue(options.selfIdKey)
                if ((selfIdValue === identity.byuId) || (selfIdValue === identity.netId))
                {
                    return true
                }
            }
            if (options.hasOwnProperty("webResourceId"))
            {
                const policies = await self.parsePolicies(options.webResourceId)
                logger("authorization policies", policies)
                for (const policy of policies)
                {
                    if (policy.informationalArea === options.policy)
                    {
                        if (options.updateType)
                        {
                            return options.updateType === policy.updateType
                        }
                        logger("user authorized without updateType")
                        return true
                    }
                }
            }
            logger("user is not authorized")
            return false
        },
        resolveToBYUId: async function (idKey)
        {
            let self = this

            const id = self.pathValue(idKey)
            let pattern = /^\d{9}$/;
            if (id.match(pattern))
            {
                return id
            }

            let idQuery = "net_id="
            idQuery += id
            let requestObject =
                {
                    url: byuapiServiceDomainURI + '/ry-api/v1.0/person/cgi/resolveIdentity.cgi/ids?' + idQuery,
                    method: 'GET',
                    json: true,
                    resolveWithFullResponse: true,
                    simple: true,
                    encoding: 'utf8',
                    headers: {
                        Accept: 'application/json'
                    }
                }

            let results = await wso2Request(requestObject)
            return results.body.ResolveIdentityService.response.byu_id
        },
        resolveToPersonId: async function (idKey)
        {
            let self = this

            const id = self.pathValue(idKey)
            let idQuery = "net_id="

            let pattern = /^\d{9}$/;
            if (id.match(pattern))
            {
                idQuery = "byu_id="
            }

            idQuery += id
            let requestObject =
                {
                    url: byuapiServiceDomainURI + '/ry-api/v1.0/person/cgi/resolveIdentity.cgi/ids?' + idQuery,
                    method: 'GET',
                    json: true,
                    resolveWithFullResponse: true,
                    simple: true,
                    encoding: 'utf8',
                    headers: {
                        Accept: 'application/json'
                    }
                }

            let results = await wso2Request(requestObject)
            return results.body.ResolveIdentityService.response.person_id
        },
        hasAccessDelegation: async function (delegatedId, fromId, category)
        {
            let requestObject =
                {
                    url: `${byuapiServiceDomainURI}/accessManagement/v1/authorization/delegation/${fromId}/${category}?asUserId=${delegatedId}`,
                    method: 'GET',
                    json: true,
                    resolveWithFullResponse: true,
                    simple: true,
                    encoding: 'utf8',
                    headers: {
                        Accept: 'application/json'
                    }
                }

            let results = await wso2Request(requestObject)
            return results.body.content
        },
        handleServerError: function (e, serverService)
        {
            this.send(500, {serverService: serverService, message: e.message, serverStatusCode: e.statusCode})
        },
        defaultErrorHandler: function (err, dumpStack)
        {
            if (dumpStack)
            {
                console.error(err.stack)
            }
            let statusCode = 500
            if (err.statusCode)
            {
                statusCode = err.statusCode
            }
            this.send(statusCode, err.message)
        },
        set: function set(property, value, defaultValue)
        {
            if (value === undefined)
            {
                if (defaultValue === undefined)
                {
                    defaultValue = this.keys[property]
                }
                if (typeof defaultValue === 'function')
                {
                    this[property] = defaultValue.call(this)
                }
                else
                {
                    this[property] = defaultValue
                }
            }
            else
            {
                this[property] = value
            }
        },
        keys: {
            lambda: false,  //must be first
            req: null,
            res: null,
            apiContext: null
            // response: this.configResponse
        }
    }

    ch.config(configObject)

    return ch
}

==> sim-client-handler-4.1.0.tgz: package/index.js <==
/*
 * Copyright 2016 Brigham Young University
 *
 * Licensed under the Apache License, Version 2.0 (the "License")
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

"use strict"

const yapi          = require("./lib/yapi")
const wso2Request   = require('byu-wso2-request').request
const logger        = require("debug")("clientHandler")

const byuapiServiceDomainURI = process.env.BYUAPI_DOMAIN || 'https://api.byu.edu'

exports.clientHandler = function(configObject)
{
    const ch =
    {
        configResponse: function ()
        {
            if (this.lambda)
            {
                return {
                    statusCode: 200,
                    headers: {
                        "content-type": "application/json"
                    },
                    body: ""
                }
            }

            return {}
        },
        config: function Config(options)
        {
            if (options === undefined) options = {}

            for (const key of Object.keys(this.keys))
            {
                this.set(key, options[key], this.keys[key])
            }
            this.response = this.configResponse()
            this.policyCache = {}
            this.identity = null
        },
        validateJWT: async function ()
        {
            let self = this

            const byuJwtFx = require('byu-jwt')
            const byuJwt = new byuJwtFx()

            const jwt = self.parseJWT()

            self.jwtDecoded = await byuJwt.decodeJWT(jwt)

            if (this.apiContext !== null && self.jwtDecoded.wso2.apiContext.indexOf(this.apiContext) !== 0 )
            {
                const err = new Error("Invalid jwt on apiContext: " + self.jwtDecoded.wso2.apiContext)
                logger(err.stack.split("\n")[1],": api context", self.jwtDecoded.wso2.apiContext, "is not found\""+this.apiContext+"\"")
                throw err
            }
        },
        send: function (statusCode, body)
        {
            if (this.lambda)
            {
                if (body === undefined)
                {
                    body = {}
                    if (statusCode >= 400)
                    {
                        let statusCodeMessage = {message: "An unknown error has occurred. Please try again."}
                        switch (statusCode)
                        {
                            case 404:
                                statusCodeMessage.message = "resource <" + this.req.resource + "> was not found"
                                break
                            case 405:
                                statusCodeMessage.message = "http Method <" + this.req.httpMethod + "> is not supported for resource <" + this.req.resource + ">"
                                break
                        }

                        body = statusCodeMessage
                    }
                }
                this.response.body = JSON.stringify(body)
                this.response.statusCode = statusCode

                return this.res.succeed(this.response)
            }

            this.response = body || {}
            this.res.status(statusCode)

            this.res.send(this.response)
        },
        body: function ()
        {
            if (this.lambda)
            {
                return JSON.parse(this.req.body)
            }
            return this.req.body
        },
        queryParams: function (asURI)
        {
            if (asURI === true)
            {
                logger("req url:", this.req.url)
                const parts = this.req.url.split("?")
                logger("req url parts:", parts, parts.length)
                if (parts.length === 2)
                {
                    logger("req query parameters:", parts[1])
                    return parts[1]
                }
                return ""
            }
            if (this.lambda)
            {
                return this.req.queryParameters
            }

            return this.req.query
        },
        pathValue: function (paramkey)
        {
            if (this.lambda)
            {
                return this.req.pathParameters[paramkey]
            }

            return this.req.params[paramkey]
        },
        parseJWT: function ()
        {
            const byuJwt = require('byu-jwt')

            let jwt = this.req.headers[byuJwt.BYU_JWT_HEADER_ORIGINAL.toLowerCase()]
            if (!jwt)
            {
                jwt = this.req.headers[byuJwt.BYU_JWT_HEADER_CURRENT.toLowerCase()]
            }
            logger("raw jwt:", jwt)

            return jwt
        },
        parseIdentity: async function ()
        {
            let self = this

            function extractIdentity(decodedJwt)
            {
                const identityPart = decodedJwt.byu || decodedJwt
                self.identity = identityPart.resourceOwner || identityPart.client
                logger("identity:", self.identity)
                return self.identity
            }

            if (self.identity !== null)
            {
                return self.identity
            }

            if (self.hasOwnProperty("jwtDecoded"))
            {
                logger("using cached decoded jwt")
                return extractIdentity(self.jwtDecoded)
            }
            else
            {
                const byuJwtFx = require('byu-jwt')
                const byuJwt = new byuJwtFx()

                const jwt = self.parseJWT()

                if (jwt)
                {
                    const decodedJwt = await byuJwt.decodeJWT(jwt)

                    self.jwtDecoded = decodedJwt
                    logger("decodedJwt:", decodedJwt)

                    return extractIdentity(decodedJwt)
                }
            }

            throw new Error("unable to establish identity")
        },
        parsePolicies: async function (webResourceId)
        {
            let self = this

            if (self.policyCache[webResourceId])
            {
                return self.policyCache[webResourceId]
            }

            const response = await yapi.userPolicies(self.identity.byuId, webResourceId)

            if (response.content.length)
            {
                self.policyCache[webResourceId] = response.content
            }
            else
            {
                self.policyCache[webResourceId] = []
            }
            return self.policyCache[webResourceId]
        },
        /**
         * options web resource id and informational area are required - updateType [U] is optional
         * @param options
         * @returns boolean
         */
        authorize: async function (options)
        {
            let self = this

            const identity = await self.parseIdentity()

            if (options.allowSelf)
            {
                const selfIdValue = self.pathValue(options.selfIdKey)
                if ((selfIdValue === identity.byuId) || (selfIdValue === identity.netId))
                {
                    return true
                }
            }
            if (options.hasOwnProperty("webResourceId"))
            {
                const policies = await self.parsePolicies(options.webResourceId)
                logger("authorization policies", policies)
                for (const policy of policies)
                {
                    if (policy.informationalArea === options.policy)
                    {
                        if (options.updateType)
                        {
                            return options.updateType === policy.updateType
                        }
                        logger("user authorized without updateType")
                        return true
                    }
                }
            }
            logger("user is not authorized")
            return false
        },
        resolveToBYUId: async function (idKey)
        {
            let self = this

            const id = self.pathValue(idKey)
            let pattern = /^\d{9}$/;
            if (id.match(pattern))
            {
                return id
            }

            let idQuery = "net_id="
            idQuery += id
            let requestObject =
                {
                    url: byuapiServiceDomainURI + '/ry-api/v1.0/person/cgi/resolveIdentity.cgi/ids?' + idQuery,
                    method: 'GET',
                    json: true,
                    resolveWithFullResponse: true,
                    simple: true,
                    encoding: 'utf8',
                    headers: {
                        Accept: 'application/json'
                    }
                }

            let results = await wso2Request(requestObject)
            return results.body.ResolveIdentityService.response.byu_id
        },
        resolveToPersonId: async function (idKey)
        {
            let self = this

            const id = self.pathValue(idKey)
            let idQuery = "net_id="

            let pattern = /^\d{9}$/;
            if (id.match(pattern))
            {
                idQuery = "byu_id="
            }

            idQuery += id
            let requestObject =
                {
                    url: byuapiServiceDomainURI + '/ry-api/v1.0/person/cgi/resolveIdentity.cgi/ids?' + idQuery,
                    method: 'GET',
                    json: true,
                    resolveWithFullResponse: true,
                    simple: true,
                    encoding: 'utf8',
                    headers: {
                        Accept: 'application/json'
                    }
                }

            let results = await wso2Request(requestObject)
            return results.body.ResolveIdentityService.response.person_id
        },
        hasAccessDelegation: async function (delegatedId, fromId, category)
        {
            let requestObject =
                {
                    url: `${byuapiServiceDomainURI}/accessManagement/v1/authorization/delegation/${fromId}/${category}?asUserId=${delegatedId}`,
                    method: 'GET',
                    json: true,
                    resolveWithFullResponse: true,
                    simple: true,
                    encoding: 'utf8',
                    headers: {
                        Accept: 'application/json'
                    }
                }

            let results = await wso2Request(requestObject)
            return results.body.content
        },
        handleServerError: function (e, serverService)
        {
            this.send(500, {serverService: serverService, message: e.message, serverStatusCode: e.statusCode})
        },
        defaultErrorHandler: function (err, dumpStack)
        {
            if (dumpStack)
            {
                console.error(err.stack)
            }
            let statusCode = 500
            if (err.statusCode)
            {
                statusCode = err.statusCode
            }
            this.send(statusCode, err.message)
        },
        set: function set(property, value, defaultValue)
        {
            if (value === undefined)
            {
                if (defaultValue === undefined)
                {
                    defaultValue = this.keys[property]
                }
                if (typeof defaultValue === 'function')
                {
                    this[property] = defaultValue.call(this)
                }
                else
                {
                    this[property] = defaultValue
                }
            }
            else
            {
                this[property] = value
            }
        },
        keys: {
            lambda: false,  //must be first
            req: null,
            res: null,
            apiContext: null
            // response: this.configResponse
        }
    }

    ch.config(configObject)

    return ch
}
